require "defines"
require "util"

require "building.structures"

local builder = {}

local d = defines.direction

function builder.getDeltaXY(size,dir)
    if dir == d.north then
        return 0, -size.height
    elseif dir == d.south then
        return 0,  size.height
    elseif dir == d.east then
        return   size.width, 0
    elseif dir == d.west then
        return  -size.width, 0
    else
        error("diagonal rows not supported")
    end
end

function builder.buildEntity(args,ghost,recipe)
	local area = {{args.position.x-0.5,args.position.y-0.5},{args.position.x+0.5,args.position.y+0.5}}
	if game.canplaceentity (args) then
		e = game.createentity (args)
		if recipe then
			e.recipe = recipe --todo station names
		end
		if ghost then
			e.die()
			local corpses = game.findentitiesfiltered {area=area, type="corpse"}
			for _,c in pairs(corpses) do
				c.destroy()
			end
		end
		return true
	end
	local e = util.findfirstentity(area, args.name)
	return e and e.direction == args.direction and (not e.getrecipe or e.recipe == recipe) --TODO other options
end

-- entity, count, pos, lineDir, size, ghost, force, buildingDir, recipe

builder.buildLineMetaTable = {
    __index = {
        ghost = config.defaultBuildGhost,
        size = {width=1,height=1},
        count = 1,
        buildingDir=d.north
    }
}

function builder.buildLine(tab)
	setmetatable(tab,buildLineMetaTable)
	local deltaX, deltaY = getDeltaXY(tab.size, tab.lineDir)
	for i=0, tab.count-1 do
		local tmp = {x=tab.pos.x+i*deltaX,y=tab.pos.y+i*deltaY}
		local args = { 
			name=tab.entity,
			position=tmp,
			direction = tab.buildingDir,
			force=tab.force
		}
		if tab.options then
			for k,v in pairs(tab.options) do args[k] = v end
		end
		buildEntity(args,tab.ghost,tab.recipe)
	end
end

function builder.canBuildLine(tab)
	setmetatable(tab,{__index={
		ghost = true,
		force = game.player.force,
		size = {width=1,height=1},
		count = 1,
		buildingDir=d.north
	}})
	local deltaX, deltaY = 0,0
	if tab.lineDir == d.north then
		deltaY = - tab.size.height
	elseif tab.lineDir == d.south then
		deltaY =   tab.size.height
	elseif tab.lineDir == d.east then
		deltaX =   tab.size.width
	elseif tab.lineDir == d.west then
		deltaX = - tab.size.width
	else
		error("diagonal rows not supported")
	end
	for i=0, tab.count-1 do
		local tmp = {x=tab.pos.x+i*deltaX,y=tab.pos.y+i*deltaY}
		local canPlace = game.canplaceentity { 
			name=tab.entity,
			position=tmp,
			direction = tab.buildingDir,
			force=tab.force
		}
		local e = util.findfirstentity({{tmp.x-0.5,tmp.y-0.5},{tmp.x+0.5,tmp.y+0.5}}, tab.entity)
		if not canPlace and (not e or e.direction ~=tab.buildingDir or (tab.recipe and e.recipe ~= tab.recipe)) then
			return false
		end
	end
	return true
end

function builder.buildStructure(struct, pos, lineDir, count)
	local comp = structures.evaluateComponents(struct, pos, lineDir, count)
	--game.makefile("tmp.txt", serpent.block(comp))
	local canPlace = false
	for _,tab in ipairs(comp) do
		canPlace = canPlace and canBuildLine(tab)
	end
	if not canPlace then
		game.player.print("Collision detected when building structure")
		return false
	end
	for _,tab in ipairs(comp) do
		buildLine(tab)
	end
	return true
end

function builder.clearArea(boundingBox, filter, force)
	for _,e in ipairs(game.findentities(boundingBox)) do
	    if not filter or filter(e) then
    		local s,e = pcall(e.orderdeconstruction, force)
    		if not s and e then
    			game.player.print(e)
    		end
		end
	end
end


return builder