require "util"

local structures = {}

local rotations = {
	none = 0,
	clockwise = 1,
	opposite = 2,
	counterclockwise = 3,
}

local function moveposition(position, direction, offset)
	if direction == defines.direction.north then
		return {x=position.x - offset.x, y=position.y - offset.y}
	elseif direction == defines.direction.south then
		return {x=position.x + offset.x, y=position.y + offset.y}
	elseif direction == defines.direction.east then
		return {x=position.x + offset.y, y=position.y - offset.x}
	elseif direction == defines.direction.west then
		return {x=position.x - offset.y, y=position.y + offset.x}
	else
		error("diagonal!")
	end
end

local function angle(dir1, dir2)
	local tmp = dir1-dir2
	if tmp > 4 then 
		return tmp-8
	elseif tmp <-4 then
		return tmp+8
	else
		return tmp
	end
end

local function rotate(dir, rot)
	if rot == rotations.none then
		return dir
	elseif rot == rotations.clockwise then
		return (dir+2)%8
	elseif rot == rotations.opposite then
		return (dir+4)%8
	elseif rot == rotations.counterclockwise then
		return (dir+6)%8
	else
		error("unknown transition")
	end
end

local structureList = {
	['refinery'] = {
		size = {width=15,height=5},
		components = {
			{entity="pipe",			 count=5,offset={x= 0,y=0},size={width=1,height=1},rotate=rotations.none},
			{entity="pipe-to-ground",count=1,offset={x= 1,y=1},size={width=1,height=5},rotate=rotations.opposite},
			{entity="pipe",			 count=5,offset={x= 2,y=0},size={width=1,height=1},rotate=rotations.none},
			{entity="pipe-to-ground",count=1,offset={x= 3,y=1},size={width=1,height=5},rotate=rotations.none},
			{entity="pipe",			 count=1,offset={x= 3,y=3},size={width=1,height=5},rotate=rotations.none},
			{entity="small-electric-pole",
									 count=1,offset={x= 3,y=2},size={width=1,height=5},rotate=rotations.none},
			{entity="oil-refinery",  count=1,offset={x= 6,y=2},size={width=5,height=5},rotate=rotations.none, recipe="advanced-oil-processing"},
			{entity="pipe",			 count=1,offset={x= 9,y=0},size={width=1,height=5},rotate=rotations.none},
			{entity="pipe-to-ground",count=1,offset={x= 9,y=2},size={width=1,height=5},rotate=rotations.opposite},
			{entity="pipe-to-ground",count=1,offset={x= 9,y=4},size={width=1,height=5},rotate=rotations.opposite},
			{entity="pipe",			 count=5,offset={x=10,y=0},size={width=1,height=1},rotate=rotations.none},
			{entity="pipe-to-ground",count=1,offset={x=11,y=2},size={width=1,height=5},rotate=rotations.none},
			{entity="pipe",			 count=5,offset={x=12,y=0},size={width=1,height=1},rotate=rotations.none},
			{entity="pipe-to-ground",count=1,offset={x=13,y=4},size={width=1,height=5},rotate=rotations.none},
			{entity="pipe",			 count=5,offset={x=14,y=0},size={width=1,height=1},rotate=rotations.none},
		},
		buildDir = function(lineDir)
			return rotate(lineDir,rotations.counterclockwise)
		end
	},
	['heavy-cracking'] = {
		size = {width=8,height=9},
		components = {
			{entity="pipe",			 count=9,offset={x= 0,y=0},size={width=1,height=1},rotate=rotations.none},
			{entity="pipe-to-ground",count=3,offset={x= 1,y=2},size={width=1,height=3},rotate=rotations.opposite},
			{entity="pipe",			 count=9,offset={x= 2,y=0},size={width=1,height=1},rotate=rotations.none},
			{entity="pipe-to-ground",count=3,offset={x= 3,y=2},size={width=1,height=3},rotate=rotations.none},
			{entity="pipe",			 count=3,offset={x= 3,y=0},size={width=1,height=3},rotate=rotations.none},
			{entity="medium-electric-pole",
									 count=1,offset={x= 3,y=4},size={width=1,height=9},rotate=rotations.none},
			{entity="chemical-plant",count=3,offset={x= 5,y=1},size={width=3,height=3},rotate=rotations.opposite, recipe="heavy-oil-cracking"},
			{entity="pipe",			 count=8,offset={x= 7,y=0},size={width=1,height=1},rotate=rotations.none},
		},
		buildDir = function(lineDir)
			return rotate(lineDir,rotations.counterclockwise)
		end
	},
	['light-cracking'] = {
		size = {width=8,height=9},
		components = {
			{entity="pipe",			 count=9,offset={x= 0,y=0},size={width=1,height=1},rotate=rotations.none},
			{entity="pipe-to-ground",count=3,offset={x= 1,y=2},size={width=1,height=3},rotate=rotations.opposite},
			{entity="pipe",			 count=9,offset={x= 2,y=0},size={width=1,height=1},rotate=rotations.none},
			{entity="pipe-to-ground",count=3,offset={x= 3,y=2},size={width=1,height=3},rotate=rotations.none},
			{entity="pipe",			 count=3,offset={x= 3,y=0},size={width=1,height=3},rotate=rotations.none},
			{entity="medium-electric-pole",
									 count=1,offset={x= 3,y=4},size={width=1,height=9},rotate=rotations.none},
			{entity="chemical-plant",count=3,offset={x= 5,y=1},size={width=3,height=3},rotate=rotations.opposite, recipe="light-oil-cracking"},
			{entity="pipe",			 count=8,offset={x= 7,y=0},size={width=1,height=1},rotate=rotations.none},
		},
		buildDir = function(lineDir)
			return rotate(lineDir,rotations.counterclockwise)
		end
	},
	['plastic'] = {
		size = {width=6,height=6},
		components = {
			{entity="logistic-chest-passive-provider",
											count=2,offset={x= 0,y=1},size={width=1,height=3},rotate=rotations.none},
			{entity="logistic-chest-requester",
											count=2,offset={x= 0,y=2},size={width=1,height=3},rotate=rotations.none, options = {requestfilters={{name="coal",index=1,count=20}}}},
			{entity="basic-inserter", 		count=2,offset={x= 1,y=1},size={width=1,height=3},rotate=rotations.none},
			{entity="basic-inserter", 		count=2,offset={x= 1,y=2},size={width=1,height=3},rotate=rotations.opposite},
			{entity="small-electric-pole",  count=1,offset={x= 1,y=3},size={width=1,height=6},rotate=rotations.none},
			{entity="chemical-plant",		count=2,offset={x= 3,y=1},size={width=3,height=3},rotate=rotations.none, recipe="plastic-bar"},
			{entity="pipe",			 		count=6,offset={x= 5,y=0},size={width=1,height=1},rotate=rotations.none},
		},
		buildDir = function(lineDir)
			return rotate(lineDir,rotations.counterclockwise)
		end
	},
	['sulfur'] = {
		size = {width=9,height=6},
		components = {
			{entity="pipe",			 count=6,offset={x= 0,y=0},size={width=1,height=1},rotate=rotations.none},
			{entity="pipe-to-ground",count=2,offset={x= 1,y=2},size={width=1,height=3},rotate=rotations.none},
			{entity="pipe",			 count=6,offset={x= 2,y=0},size={width=1,height=1},rotate=rotations.none},
			{entity="pipe-to-ground",count=2,offset={x= 3,y=2},size={width=1,height=3},rotate=rotations.opposite},
			{entity="pipe",			 count=2,offset={x= 3,y=0},size={width=1,height=3},rotate=rotations.none},
			{entity="chemical-plant",count=2,offset={x= 5,y=1},size={width=3,height=3},rotate=rotations.none, recipe="sulfur"},
			{entity="basic-inserter",count=2,offset={x= 7,y=1},size={width=1,height=3},rotate=rotations.none},
			{entity="small-electric-pole",    
									 count=1,offset={x= 7,y=3},size={width=1,height=6},rotate=rotations.none},
			{entity="logistic-chest-passive-provider", 
									 count=2,offset={x= 8,y=1},size={width=1,height=3},rotate=rotations.none},
		},
		buildDir = function(lineDir)
			return rotate(lineDir,rotations.clockwise)
		end
		
	},
	['smelter'] = {
		
	},
}

function structures.getStructureSize(name, lineDir, count)
	local width,height 
	if lineDir == defines.direction.north or lineDir == defines.direction.south then
		width  = structureList[name].size.width
		height = structureList[name].size.height
		return {width = width, height = height*count}
	else -- switching width and height
		width  = structureList[name].size.height
		height = structureList[name].size.width
		return {width = width*count, height = height}
	end	
end

function structures.evaluateComponents(struct,pos,lineDir,count)
	if lineDir % 2 ~=0 then
		error("diagonal structures not supported")
	end
	local s = structures[struct]
	local buildDir = s.buildDir and s.buildDir(lineDir) or lineDir
	local ret = {}
	for _,comp in pairs(s.components) do
		local tmp = {}
		tmp.entity = comp.entity
		if comp.recipe then tmp.recipe = comp.recipe end
		tmp.size={}
		if lineDir == defines.direction.north or lineDir == defines.direction.south then
			tmp.size.width  = comp.size.width
			tmp.size.height = comp.size.height
		else -- switching width and height
			tmp.size.width  = comp.size.height
			tmp.size.height = comp.size.width
		end
		tmp.pos = moveposition(pos, lineDir, comp.offset)
		tmp.count = count*comp.count
		tmp.lineDir = lineDir
		tmp.buildingDir = rotate(buildDir, comp.rotate)
		tmp.options = table.deepcopy(comp.options)
		ret[_] = tmp
	end
	return ret
end

return structures