require "utility.util"

local oreq = require

function require(path)
    --trace(nil,path)
    --saveLog("require.txt")
    return oreq(path)
end

require "defines"

require "config"
require "gui"

local builder = require "building.builder"
local generation = require "building.generation"
eventLoop = require "event_loop"

local function loadGenerators()
    for _,file in pairs(config.generators) do
        --debugPrint("require generators."..file)
        gui.addGenerator( require ("generators."..file) ) -- () important 
    end
end
loadGenerators()

local function reactivateMeta(table)
    if table.metatable then
       setmetatable(table, table.metatable) 
       debugPrint("Reactivating: "..serpent.block(table))
    end
    for k,value in pairs(table) do
        if type(value) == "table" then
            local s,e  = pcall (reactivateMeta, value) -- stack over flow ???
            trace(not s)
        end
    end
end

game.onload(function()
    debugPrint("onload")
    --reactivateMeta(glob)
    debugPrint(glob)
    saveLog("glob.lua")
    gui.updateGui()
end)

game.oninit(function()
    debugPrint("oninit")
    reactivateMeta(glob)
    gui.updateGui()
end)

game.onevent(defines.events.ontick, function(event)
    eventLoop.resume()
end)

game.onevent(defines.events.onresearchfinished, function(event)
    if event.research ~= nil and event.research.name == config.unlockResearch then
        gui.updateGui()
    end
end)

local function calcPosition(pos,offset)
    if not pos then pos = game.player.position end
    pos.x = math.floor(pos.x) + 0.5
    pos.y = math.floor(pos.y) + 0.5
    if offset then
        pos.x = pos.x + offset.x
        pos.y = pos.y + offset.y
    end
    return pos
end


remote.addinterface("builder", {
    line = function(tab, offset)
        tab.pos = calcPosition(tab.pos, offset)
        if not builder.buildLine(tab) then
            error("Unable to place Structure")
        end
    end,
    --struct, lineDir, count, pos, offset
    structure = function(tab)
        local pos = calcPosition(tab.pos, tab.offset)
        if not builder.buildStructure(tab.struct, pos, tab.lineDir, tab.count) then
            error("Unable to place Structure")
        end
    end,
    clear = function(pos,size,filter)
		debugPrint(filter.." , isNil: "..tostring(not filter));
        if filter and type(filter) == 'string' then
            filter = lookupFilter(filter)
        end
		
        builder.clearArea({{pos.x-size,pos.y-size},{pos.x+size,pos.y+size}},filter)
    end,
    destroyArea = function(pos,size,filter)
        if type(filter) == 'string' then
            filter = lookupFilter(filter)
        end
        for _,e in ipairs(game.findentities({{pos.x-size,pos.y-size},{pos.x+size,pos.y+size}})) do
            if not filter or filter(e) then
                local s,err = pcall(e.orderdeconstruction, game.player.force)
                if not s and e.type ~= "player" then
                    e.destroy()
                end
            end
        end
    end,
--    generateOilField = function ()
--        local ent, pos = findOilWells()
--        local positions = {}
--        for _,res in pairs(ent) do
--            positions[#positions+1] = res.position
--        end
--        eventLoop.add(generation.connectOilWells, positions,pos)
--        --game.makefile("polyLog.txt",log); log=""
--    end,
})
