local event_loop = {}

local List = {}
List.__index = List

function List.new ()
    return setmetatable({first = 0, last = -1},List)
end

function List:push(value)
    local last = self.last + 1
    self.last = last
    self[last] = value
end

function List:pop()
    local first = self.first
    if first > self.last then return nil end
    local value = self[first]
    self[first] = nil        -- to allow garbage collection
    self.first = first + 1
    return value
end

function List:isEmpty()
    return self.first > self.last
end

local function pack2(...) return {n=select('#', ...), ...} end
local function unpack2(t) return unpack(t, 1, t.n) end

local runnables = List.new()

local function loop()
    while true do
        if not runnables:isEmpty() then
            local next = runnables:pop()
            next.func(unpack2(next.params))
        end
        coroutine:yield()
    end
end


function event_loop.add(func,...)
    if not func or type(func)~="function" then
        error("not a function: "..func)
    end
    runnables:push({func=func,params=pack2(...)})
end

event_loop.resume = coroutine.wrap(loop)

return event_loop