gui = {}

local selectors = require("utility.selector");

local generators = {}

--Poly states: Start Selection, Finish, Reset

local option2create = {
    Polygon = function(player, table, i, option)
        table.add {type="label", caption = option.name..": "}
        local button = table.add {type="button", name="opt_"..i}
        local state = getData(player, "optionState", i)
        if state == nil then
            button.caption="Start Selection"
        elseif state.type=="selector" then
            button.caption="Finish Selection"
        elseif state.type == "polygon" then
            button.caption="Reset Selection"
        end
    end,
    Position = function(player, table, i, option)
        table.add {type="label", caption = option.name..": "}
        local button = table.add {type="button", name="opt_"..i}
        local state = getData(player, "optionState", i)
        if state == nil then
            button.caption="Start Selection"
        elseif state.type=="selector" then
            button.caption="Finish Selection"
        elseif state.type == "point" then
            button.caption="Reset Selection"
        end
    end,
    checkbox = function(player, table, i, option)
        local state = default(getData(player,"optionState",i), option.selected)
        table.add {type="checkbox", name="opt_"..i, caption=option.name, state=state}
        table.add {type="label", caption = ""}
        setData(player, "optionState", i, state)
    end,
    textfield = function(player, table, i, option)
        table.add {type="label", caption = option.name..": "}
        local field = table.add {type="textfield", name="opt_"..i}
        field.text = default(getData(player,"optionState",i), option.value)
        setData(player, "optionState", i, field.text)
    end,
    Option = function(player, table, i, option)
        table.add {type="label", caption = option.name..": "}
        local sel = default(getData(player,"optionState",i), option.selected)
        local button = table.add {type="button", name="opt_"..i, caption=option.values[sel]}
        setData(player, "optionState", i, sel)
    end,
}

local function finishSelection(player)
    local selector = getData(player, "activeSelector")
    if selector then
        local data = selector:finish(player)
        
        if selector:isComplete() then
            setData(player, "optionState", selector.index, data)
            player.gui.left.expansion.rows["opt_"..(selector.index)].caption = "Reset Selection"
        else
            setData(player, "optionState", selector.index, nil)
            player.gui.left.expansion.rows["opt_"..(selector.index)].caption = "Start Selection"
        end
        setData(player, "activeSelector", nil)
    end
    return selector
end

local function toggleSelectionState(player, i, selectorCreator)
    local selector = finishSelection(player)
    if selector and selector.index == i then
        return -- nothing left todo
    end
    
    local state = getData(player, "optionState", i)
    if not state then
        selector = selectorCreator(getData(player,"optionState",i))
        selector.index = i
        setData(player,"optionState", i, selector)
        setData(player, "activeSelector", selector)
        player.gui.left.expansion.rows["opt_"..i].caption = "Finish Selection"
    elseif state.type ~= "selector" then
        setData(player,"optionState", i, nil)
        player.gui.left.expansion.rows["opt_"..i].caption = "Start Selection"
    end
end

local option2onClick = {
    Polygon = function(element, player, i, option)
        toggleSelectionState(player, i, selectors.createPolygonSelector)
    end,
    Position = function(element, player, i, option)
        toggleSelectionState(player, i, selectors.createPointSelector)
    end,
    checkbox = function(element, player, i, option)
        setData(player,"optionState",i, element.state)
    end,
    textfield = function(element, player, i, option)
        setData(player, "optionState", i, element.text)
    end,
    Option = function(element, player, i, option)
        local state = getData(player, "optionState", i)
        state = (state % #option.values) + 1
        setData(player, "optionState", i, state)
        element.caption = option.values[state]
    end,
}

game.onevent(defines.events.onbuiltentity, function(event)
    if event.createdentity.name == "land-mark" then
        local player = game.getplayer(event.playerindex)
        local selector = getData(player,"activeSelector")
        if selector then
            selector:addMarker(event.createdentity)
        else
            player.print("No position selection active!")
        end
    end
end)

game.onevent(defines.events.onguiclick, function(event)
    local index = event.playerindex or event.name
    local player = game.getplayer(index)
    local name = event.element.name
    if name:find("opt_") then
        local index = tonumber(name:sub(5,-1))
        local generator = generators[getData(player, "selectedGenerator")]
        local option = generator.options[index]
        debugPrint("OnClick called for "..index.." with type "..default(option,{type="nil"}).type)
        option2onClick[option.type](event.element, player, index, option)        
    elseif name:find("start_") then 
        local generatorName = name:sub(7,-1)
        local generator = generators[generatorName]
        gui.createGui(player, generator)
    elseif name:find("build") then
        local generator = generators[getData(player,"selectedGenerator")]
        local options = {}
        finishSelection(player) -- close selector if any open
        for i, opt in ipairs(generator.options) do
            local data = getData(player, "optionState", i)
            if opt.type == "textfield" then
                data = player.gui.left.expansion.rows["opt_"..i].text
            end
            options[i] = data;
            options[opt.name] = data;
        end
        eventLoop.add(generator.build, player, options)
        --generator.build(player, options)
        gui.createGui(player,nil)
    elseif name:find("cancel") then
        gui.createGui(player,nil)
    end
end)

function gui.destroyGui(player)
    if player.gui.left.expansion ~= nil then
        player.gui.left.expansion.destroy()
    end
end

function gui.createGui(player, generator)
    if player.gui.left.expansion ~= nil then
        player.gui.left.expansion.destroy()
    end
    if generator then
        local lastGen = setData(player, "selectedGenerator", generator.name)
        if not lastGen or lastGen ~= generator.name then
            setData(player, "optionState", {}) -- reset optionState table
        end
        
        local expansion = player.gui.left.add {type="frame", direction="vertical",
             name="expansion", caption = generator.name}
        local table = expansion.add {type="table", name="rows", colspan=2}
        
        for i,option in ipairs(generator.options) do
            option2create[option.type](player, table, i, option)
        end
        
        table.add{type="button", name="cancel", caption="Cancel"}
        table.add{type="button", name="build", caption="Build"}
    else
        setData(player, "optionState", {}) -- reset optionState table
        setData(player,"selectedGenerator", nil)
        
        local expansion = player.gui.left.add {type="frame", direction="vertical",
             name="expansion", caption = "Start Generator"}
        local rows = expansion.add {type="table", name="rows", colspan=1}
        for name,gen in pairs(generators) do
            rows.add {type="button", name="start_"..gen.name, caption=gen.name}
        end
    end
end

function gui.updateGui()
    for index, player in pairs(game.players) do
        if player.force.technologies[config.unlockResearch].researched then
            gui.createGui(player, generators[getData(player, "selectedGenerator")])
        end
    end
end

function gui.addGenerator(g)
    generators[g.name] = g;
end

