--library classes for pathfinding
local Grid = require ("jumper.grid")
local Node = require ("jumper.core.node")

local Map = setmetatable({},Grid)
Map.__index = Map
--[[
Map.__call = function(self,x,y)
    return self:getNodeAt(x,y)
end]]
Map.__call = function()
    return Map.new()
end

Map.tileStates = {
    empty = 0,
    well = 1,
    pipe = 2,
    obstacle = 3,
    belt = 4,
    pole = 5,
    miner = 6,
    factory = 7
}

function Map.new()
    local self = setmetatable({},Map)
    self.metatable = Map -- restore after reload
    self.nodes = {}
    self.map = {}
    self.width = 0
    self.height = 0
    return self
end

-- Gets the node at location <x,y> on a postprocessed grid
function Map:getNodeAt(x,y)
    if not x or not y then return end
    if x < self.min_bound_x or x > self.max_bound_x then return end
    if y < self.min_bound_y or y > self.max_bound_y then return end
    if not self.nodes[y] then self.nodes[y] = {} end
    if not self.nodes[y][x] then self.nodes[y][x] = Node:new(x,y) end
    return self.nodes[y][x]
end

function Map:setTile(x, y, tile)
    self:set(x,y,{state=tile})
end

function Map:set(x, y, val)
    if type(val) ~= 'table' or not val.state then
        error("val not a valid table")
    end
    if not self.map[y]  then
        self.map[y] = {}
    end
    self.map[y][x] = val
    if self.min_bound_x == nil then
        self.min_bound_x, self.max_bound_x = x,x
        self.min_bound_y, self.max_bound_y = y,y
    else
        self.min_bound_x = math.min(self.min_bound_x,x)
        self.max_bound_x = math.max(self.max_bound_x,x)
        self.min_bound_y = math.min(self.min_bound_y,y)
        self.max_bound_y = math.max(self.max_bound_y,y)
        self.width  = (self.max_bound_x-self.min_bound_x)+1
        self.height = (self.max_bound_y-self.min_bound_y)+1
    end
end

function Map:addPipe(pos, id)
    self:set(pos.x,pos.y, {state = Map.tileStates.pipe, id = id})
end

function Map:addWell(well)
    for i= -1, 1 do
        for j= -1, 1 do
            self:set(well.x+i, well.y+j, {state = Map.tileStates.well, well = well.id})
        end
    end
end

function Map:addObstacle(pos)
    self.setTile(pos.x,pos.y, Map.tileStates.obstacle)
end


function Map:getTileState(x,y)
    if self.map[y] and self.map[y][x] then
        return self.map[y][x].state
    end
    return Map.tileStates.empty
end

function Map:getTile(x,y)
    if self.map[y] then
        return self.map[y][x]
    end
    return nil
end    

function Map:getBoundingBox()
    return {{x=self.min_bound_x,y=self.min_bound_y},{x=self.max_bound_x,y=self.max_bound_y}}
end

function Map:isWalkableAt(x, y, walkable)
    if not walkable then return true end
    local nodeValue = self:getTileState(x,y)
    
    if self.__eval then return walkable(nodeValue) end
    return (nodeValue == walkable)
end

function Map:save()
    game.makefile("map.lua",serpent.block(self))
end

function Map:clear()
    self.nodes = {}
    self.map = {}
    self.width = 0
    self.height = 0
    self.min_bound_x, self.max_bound_x = nil,nil
    self.min_bound_y, self.max_bound_y = nil,nil
end

return Map
