local Polygon = {}
Polygon.__index = Polygon


function Polygon.new(p)
    local self = setmetatable({},Polygon)
    self.metatable = Polygon -- restore after reload
    self.type = "polygon"
    self.points = {}
    self.points[1] = p
    self.xMin = p.x
    self.xMax = p.x
    self.yMin = p.y
    self.yMax = p.y
    return self;
end

function Polygon:addPoint(p)
    self.points[#self.points+1] = p
    if p.x < self.xMin then self.xMin = p.x end
    if p.x > self.xMax then self.xMax = p.x end
    if p.y < self.yMin then self.yMin = p.y end
    if p.y > self.yMax then self.yMax = p.y end
end

function Polygon:contains(p)
    -- Boundingbox check
    if (p.x > self.xMax or p.x < self.xMin) or (p.y > self.yMax or p.y < self.yMin) then
        return false
    end
    -- inside Bounding box, do some ray tracing
    local sp = self.points
    local c = false
    local j = #self.points
    for i = 1, #self.points  do
        if ((sp[i].y>p.y) ~= (sp[j].y>p.y)) and
            (p.x < (sp[j].x-sp[i].x) * (p.y-sp[i].y) / (sp[j].y-sp[i].y) + sp[i].x) then
            c = not c
        end
        j=i
    end
    return c;
end

function Polygon:findEntities(name,type)
    local area = {{self.xMin, self.yMin}, {self.xMax, self.yMax}}
    local ent = game.findentitiesfiltered{area = area, name=name, type = type}
    local ret = {}
    for _,e in pairs(ent) do
        if self:contains(e.position) then ret[#ret+1] = e end
    end
    return ret
end

function Polygon:print()
    for i,p in ipairs(self.points) do
        debugPrint(("%d: [%d,%d]"):format(i, p.x, p.y))
    end
end

function Polygon:isComplete()
    return #(self.points) >= 3 -- at least 3 points
end

return Polygon