local Polygon = require "utility.polygon"

local Selector = {}
Selector.__index = Selector

local function deleteMarker(player, markers)
    local count = 0;
    for _,e in pairs(markers) do
        if e.valid then
            e.destroy()
            count = count + 1
        end
    end
    if count > 0 then
        player.insert {name="marker", count = count}
    end
end

function Selector:addMarker(entity)
    self.markers[#self.markers+1] = entity
    if self.select == "polygon" then
        if not self.data then
            self.data = Polygon.new(entity.position)
        else
            self.data:addPoint(entity.position)
        end
    elseif self.select == "point" then
        self.data = entity.position
    end
end

function Selector:isComplete()
    return self.data ~= nil and (not self.data.isComplete or self.data:isComplete())
end

function Selector:finish(player)
    deleteMarker(player, self.markers)
    return self.data
end

function Selector.createRawSelector(type, data)
    if data and data.type == "selector" then
        return data
    end
    local self = setmetatable({},Selector)
    self.metatable = Selector -- restore after reload
    self.type = "selector"
    self.select = type
    self.data = data
    self.markers = {}
    return self
end

function Selector.createPointSelector(pos)
    return Selector.createRawSelector("point", pos)
end

function Selector.createPolygonSelector(poly)
    return Selector.createRawSelector("polygon", poly)
end

return Selector