function trace(condition, string)
    if condition == nil or condition then
        local s = debug.traceback(string,2)
        for line in s:gmatch("[^\n]+") do
            debugPrint(line)
        end
        game.makefile("lasttrace.txt",s)
    end
end

local log = ""
function debugPrint(string)
    if type(string) == "table" then
        string = serpent.block(string)
    elseif type(string) == "boolean" then
        if string then
            string = "true"
        else
            string = "false"
        end
    end
    for index, player in pairs(game.players) do
        player.print(string)
    end
    log = log..string.."\n"
end

function clearLog()
    log = ""
end

function saveLog(file)
    game.makefile(file,log)
    log = ""
end

function lookupFilter(filterName)
    filterName = filterName:lower()
    if filterName == "tree" or filterName == "trees" then
        return function(e)
            return e.type == "tree"
        end
    end
    if filterName == "rock" or filterName == "rocks" then
        return function(e)
            return e.name == "stone-rock"
        end
    end
    if filterName == "enemy" or filterName == "enemies" then
        return function(e) -- FIXME
            return e.force.name == game.forces.enemy.name
        end
    end
    return loadstring(filterName) -- experimental
end

function default(value, def)
    if value then
        return value
    else
        return def
    end
end

function setData(player, ...)
    if not glob[player.name] then
        glob[player.name] = {}
    end
    local table = glob[player.name]
    local count = select("#", ...);
    if count > 2 then
        for n=1, count-2 do
            local e = select(n, ...)
            if not table[e] then
                table[e] = {}
            end
            table = table[e]
        end
    elseif count < 2 then
        error "not enough arguments"
    end
    local old = table[select(count-1)]
    if not select(count-1, ...) then
        trace(nil, serpent.block{...})
        return old
    end
    table[select(count-1, ...)] = select(count, ...)
    return old
end

function getData(player, ...)
    trace(not player, "no player")
    if not glob[player.name] then
        return nil
    end
    local table = glob[player.name]
    local count = select("#",...);
    if count > 1 then
        for n=1, count-1 do
            local e = select(n,...)
            if not table[e] then
                return nil
            end
            table = table[e]
        end
    elseif count < 1 then
        error "not enough arguments"
    end
    return table[select(count, ...)];
end
